# -*- coding: utf-8 -*-

"""
@author: <sylvain.herledan@oceandatalab.com>
@date: 2016-09-07
"""

import flatbuffers
import SEAScope.API.Collection
import SEAScope.types.renderable_id
import SEAScope.types.tag
import SEAScope.types.variable


def serialize(builder, collection_obj):
    """"""
    if builder is None:
        builder = flatbuffers.Builder(0)

    cls = SEAScope.API.Collection

    _must_be_current = collection_obj['mustBeCurrent']
    _x_seamless = collection_obj['xSeamless']
    _y_seamless = collection_obj['ySeamless']
    _label = builder.CreateString(collection_obj['label'])

    tags = []
    l = len(collection_obj['tags'].keys())
    for k, v in collection_obj['tags'].items():
        _, tag = SEAScope.types.tag.serialize(builder, k, v)
        tags.append(tag)
    cls.CollectionStartTagsVector(builder, l)
    for t in tags:
        builder.PrependUOffsetTRelative(t)
    _tags = builder.EndVector(l)

    variables = []
    l = len(collection_obj['variables'])
    for var_obj in collection_obj['variables']:
        _, variable = SEAScope.types.variable.serialize(builder, var_obj)
        variables.append(variable)
    cls.CollectionStartVariablesVector(builder, l)
    for v in variables:
        builder.PrependUOffsetTRelative(v)
    _variables = builder.EndVector(l)

    _default_variable = collection_obj['defaultVariable']
    _, _rid = SEAScope.types.renderable_id.serialize(builder,
                                                     collection_obj['id'])

    cls.CollectionStart(builder)
    cls.CollectionAddId(builder, _rid)
    cls.CollectionAddMustBeCurrent(builder, _must_be_current)
    cls.CollectionAddXSeamless(builder, _x_seamless)
    cls.CollectionAddYSeamless(builder, _y_seamless)
    cls.CollectionAddLabel(builder, _label)
    cls.CollectionAddTags(builder, _tags)
    cls.CollectionAddVariables(builder, _variables)
    cls.CollectionAddDefaultVariable(builder, _default_variable)
    collection = cls.CollectionEnd(builder)

    return builder, collection


def deserialize(buf):
    """"""
    col_obj = {}
    col_obj['id'] = SEAScope.types.renderable_id.deserialize(buf.Id())
    col_obj['label'] = buf.Label()
    col_obj['mustBeCurrent'] = 0 < buf.MustBeCurrent()
    col_obj['xSeamless'] = 0 < buf.XSeamless()
    col_obj['ySeamless'] = 0 < buf.YSeamless()
    col_obj['tags'] = {}
    tags_count = buf.TagsLength()
    for i in range(0, tags_count):
        name, value = SEAScope.types.tag.deserialize(buf.Tags(i))
        col_obj['tags'][name] = value

    col_obj['variables'] = []
    vars_count = buf.VariablesLength()
    for i in range(0, vars_count):
        v = SEAScope.types.variable.deserialize(buf.Variables(i))
        col_obj['variables'].append(v)

    return col_obj
