import sys
import struct
import socket
import logging
import contextlib
import SEAScope.cmds.add_collection
import SEAScope.cmds.add_variable
import SEAScope.cmds.add_granule
import SEAScope.cmds.set_rendering
import SEAScope.cmds.get_rendering
import SEAScope.cmds.set_current_datetime
import SEAScope.cmds.set_altitude
import SEAScope.cmds.look_at
import SEAScope.cmds.zoom_in
import SEAScope.cmds.zoom_out
import SEAScope.cmds.reset_camera

logger = logging.getLogger(__name__)


@contextlib.contextmanager
def connect(host, port):
    """Return an open socket which can be used in a "with" statement."""
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, port))
        yield s
    except:
        _, e, _ = sys.exc_info()
        logger.error(e)
        raise
    finally:
        s.close()


def read_response(link, msg_size):
    """"""
    chunk_size = msg_size
    already_read = 0
    result = b''
    while already_read < msg_size:
        buf = link.recv(chunk_size)
        already_read += len(buf)
        result += buf
        chunk_size = msg_size - already_read
    return result


def collection(s, _collection):
    """"""
    serializer = SEAScope.cmds.add_collection.serialize
    builder = None
    builder, serialized = serializer(builder, _collection)
    builder.Finish(serialized)
    buf = builder.Output()
    s.sendall(struct.pack('>Q', len(buf)) + buf)


def variable(s, collection_id, _variable):
    """"""
    serializer = SEAScope.cmds.add_variable.serialize
    builder = None
    builder, serialized = serializer(builder, collection_id, _variable)
    builder.Finish(serialized)
    buf = builder.Output()
    s.sendall(struct.pack('>Q', len(buf)) + buf)


def rendering_config(s, rcfg):
    """"""
    serializer = SEAScope.cmds.set_rendering.serialize
    builder = None
    builder, serialized = serializer(builder, rcfg)
    builder.Finish(serialized)
    buf = builder.Output()
    s.sendall(struct.pack('>Q', len(buf)) + buf)


def granule(s, _granule):
    """"""
    serializer = SEAScope.cmds.add_granule.serialize
    builder = None
    builder, serialized = serializer(builder, _granule)
    builder.Finish(serialized)
    buf = builder.Output()
    s.sendall(struct.pack('>Q', len(buf)) + buf)


def current_datetime(link, timestamp):
    """"""
    serializer = SEAScope.cmds.set_current_datetime.serialize
    builder = None
    builder, serialized = serializer(builder, timestamp)
    builder.Finish(serialized)
    buf = builder.Output()
    link.sendall(struct.pack('>Q', len(buf)) + buf)


def rendering_config_for(link, target):
    """"""
    # Request rendering config from SEAScope
    serializer = SEAScope.cmds.get_rendering.serialize
    deserializer = SEAScope.cmds.get_rendering.deserialize
    builder = None
    builder, serialized = serializer(builder, target)
    builder.Finish(serialized)
    buf = builder.Output()
    link.sendall(struct.pack('>Q', len(buf))+buf)

    # Retrieve result
    buf = link.recv(8)
    l = struct.unpack('<Q', buf)
    buf = read_response(link, l[0])
    result = deserializer(buf)
    return result

def altitude(link, altitude):
    """"""
    serializer = SEAScope.cmds.set_altitude.serialize
    builder = None
    builder, serialized = serializer(builder, altitude)
    builder.Finish(serialized)
    buf = builder.Output()
    link.sendall(struct.pack('>Q', len(buf))+buf)

def location(link, lon, lat):
    """"""
    serializer = SEAScope.cmds.look_at.serialize
    builder = None
    builder, serialized = serializer(builder, lon, lat)
    builder.Finish(serialized)
    buf = builder.Output()
    link.sendall(struct.pack('>Q', len(buf))+buf)

def zoom_in(link):
    """"""
    serializer = SEAScope.cmds.zoom_in.serialize
    builder = None
    builder, serialized = serializer(builder)
    builder.Finish(serialized)
    buf = builder.Output()
    link.sendall(struct.pack('>Q', len(buf))+buf)

def zoom_out(link):
    """"""
    serializer = SEAScope.cmds.zoom_out.serialize
    builder = None
    builder, serialized = serializer(builder)
    builder.Finish(serialized)
    buf = builder.Output()
    link.sendall(struct.pack('>Q', len(buf))+buf)

def reset_camera(link):
    """"""
    serializer = SEAScope.cmds.reset_camera.serialize
    builder = None
    builder, serialized = serializer(builder)
    builder.Finish(serialized)
    buf = builder.Output()
    link.sendall(struct.pack('>Q', len(buf))+buf)
